@@ -48,6 +48,7 @@
 #include "empathy-contact-list-store.h"
 #include "empathy-contact-list-view.h"
 #include "empathy-contact-menu.h"
+#include "empathy-search-bar.h"
 #include "empathy-theme-manager.h"
 #include "empathy-smiley-manager.h"
 #include "empathy-ui-utils.h"
@@ -93,8 +94,12 @@ typedef struct {
 	GtkWidget         *label_topic;
 	GtkWidget         *contact_list_view;
 	GtkWidget         *info_bar_vbox;
+	GtkWidget         *search_bar;
 
 	guint              unread_messages;
+	/* TRUE if the pending messages can be displayed. This is to avoid to show
+	 * pending messages *before* messages from logs. (#603980) */
+	gboolean           can_show_pending;
 } EmpathyChatPriv;
 
 typedef struct {
@@ -205,6 +210,43 @@ chat_connect_channel_reconnected (EmpathyDispatchOperation *dispatch,
 	}
 }
 
+static void
+reconnected_connection_ready_cb (TpConnection *connection,
+			const GError *error,
+			gpointer user_data)
+{
+	EmpathyChat *chat = user_data;
+	EmpathyChatPriv *priv = GET_PRIV (chat);
+
+	if (error != NULL) {
+		DEBUG ("connection is not ready: %s", error->message);
+		goto out;
+	}
+
+	DEBUG ("Account reconnected, request a new Text channel");
+
+	switch (priv->handle_type) {
+		case TP_HANDLE_TYPE_CONTACT:
+			empathy_dispatcher_chat_with_contact_id (
+				connection, priv->id,
+				chat_connect_channel_reconnected,
+				chat);
+			break;
+		case TP_HANDLE_TYPE_ROOM:
+			empathy_dispatcher_join_muc (connection,
+				priv->id,
+				chat_connect_channel_reconnected,
+				chat);
+			break;
+		default:
+			g_assert_not_reached ();
+			break;
+	}
+
+out:
+	g_object_unref (chat);
+}
+
 static void
 chat_new_connection_cb (TpAccount   *account,
 			guint        old_status,
@@ -217,32 +259,19 @@ chat_new_connection_cb (TpAccount   *account,
 	EmpathyChatPriv *priv = GET_PRIV (chat);
 	TpConnection *connection;
 
+	if (new_status != TP_CONNECTION_STATUS_CONNECTED)
+		return;
+
 	connection = tp_account_get_connection (account);
 
-	if (!priv->tp_chat && account == priv->account &&
-	    priv->handle_type != TP_HANDLE_TYPE_NONE &&
-	    !EMP_STR_EMPTY (priv->id)) {
+	if (priv->tp_chat != NULL || account != priv->account ||
+	    priv->handle_type == TP_HANDLE_TYPE_NONE ||
+	    EMP_STR_EMPTY (priv->id))
+		return;
 
-		DEBUG ("Account reconnected, request a new Text channel");
-
-		switch (priv->handle_type) {
-			case TP_HANDLE_TYPE_CONTACT:
-				empathy_dispatcher_chat_with_contact_id (
-					connection, priv->id,
-					chat_connect_channel_reconnected,
-					chat);
-				break;
-			case TP_HANDLE_TYPE_ROOM:
-				empathy_dispatcher_join_muc (connection,
-					priv->id,
-					chat_connect_channel_reconnected,
-					chat);
-				break;
-			default:
-				g_assert_not_reached ();
-				break;
-		}
-	}
+	g_object_ref (chat);
+	tp_connection_call_when_ready (connection, reconnected_connection_ready_cb,
+				   chat);
 }
 
 static void
@@ -591,7 +620,7 @@ chat_command_join_cb (EmpathyDispatchOperation *dispatch,
 	if (error != NULL) {
 		DEBUG ("Error: %s", error->message);
 		empathy_chat_view_append_event (chat->view,
-			_("Failed to join chatroom"));
+			_("Failed to join chat room"));
 	}
 }
 
@@ -668,13 +697,24 @@ static void
 chat_command_join (EmpathyChat *chat,
 		   GStrv        strv)
 {
+	guint i = 0;
 	EmpathyChatPriv *priv = GET_PRIV (chat);
-	TpConnection *connection;
 
-	connection = empathy_tp_chat_get_connection (priv->tp_chat);
-	empathy_dispatcher_join_muc (connection, strv[1],
-				     chat_command_join_cb,
-				     chat);
+	GStrv rooms = g_strsplit_set (strv[1], ", ", -1);
+
+	while (rooms[i] != NULL) {
+		/* ignore empty strings */
+		if (!EMP_STR_EMPTY (rooms[i])) {
+			TpConnection *connection;
+
+			connection = empathy_tp_chat_get_connection (priv->tp_chat);
+			empathy_dispatcher_join_muc (connection, rooms[i],
+						     chat_command_join_cb,
+						     chat);
+		}
+		i++;
+	}
+	g_strfreev (rooms);
 }
 
 static void
@@ -772,36 +812,36 @@ typedef struct {
 
 static ChatCommandItem commands[] = {
 	{"clear", 1, 1, chat_command_clear,
-	 N_("/clear, clear all messages from the current conversation")},
+	 N_("/clear: clear all messages from the current conversation")},
 
 	{"topic", 2, 2, chat_command_topic,
-	 N_("/topic <topic>, set the topic of the current conversation")},
+	 N_("/topic <topic>: set the topic of the current conversation")},
 
 	{"join", 2, 2, chat_command_join,
-	 N_("/join <chatroom id>, join a new chatroom")},
+	 N_("/join <chat room ID>: join a new chat room")},
 
 	{"j", 2, 2, chat_command_join,
-	 N_("/j <chatroom id>, join a new chatroom")},
+	 N_("/j <chat room ID>: join a new chat room")},
 
 	{"query", 2, 3, chat_command_query,
-	 N_("/query <contact id> [<message>], open a private chat")},
+	 N_("/query <contact ID> [<message>]: open a private chat")},
 
 	{"msg", 3, 3, chat_command_msg,
-	 N_("/msg <contact id> <message>, open a private chat")},
+	 N_("/msg <contact ID> <message>: open a private chat")},
 
 	{"nick", 2, 2, chat_command_nick,
-	 N_("/nick <nickname>, change your nickname on current server")},
+	 N_("/nick <nickname>: change your nickname on the current server")},
 
 	{"me", 2, 2, chat_command_me,
-	 N_("/me <message>, send an ACTION message to the current conversation")},
+	 N_("/me <message>: send an ACTION message to the current conversation")},
 
 	{"say", 2, 2, chat_command_say,
-	 N_("/say <message>, send <message> to the current conversation. "
+	 N_("/say <message>: send <message> to the current conversation. "
 	    "This is used to send a message starting with a '/'. For example: "
-	    "\"/say /join is used to join a new chatroom\"")},
+	    "\"/say /join is used to join a new chat room\"")},
 
 	{"help", 1, 2, chat_command_help,
-	 N_("/help [<command>], show all supported commands. "
+	 N_("/help [<command>]: show all supported commands. "
 	    "If <command> is defined, show its usage.")},
 };
 
@@ -961,7 +1001,7 @@ chat_send (EmpathyChat  *chat,
 
 		if (!second_slash) {
 			empathy_chat_view_append_event (chat->view,
-				_("Unknown command, see /help for the available"
+				_("Unknown command; see /help for the available"
 				  " commands"));
 			return;
 		}
@@ -1243,6 +1283,13 @@ chat_input_text_buffer_changed_cb (GtkTextBuffer *buffer,
 	}
 }
 
+static gboolean
+empathy_isspace_cb (gunichar c,
+		 gpointer data)
+{
+	return g_unichar_isspace (c);
+}
+
 static gboolean
 chat_input_key_press_event_cb (GtkWidget   *widget,
 			       GdkEventKey *event,
@@ -1324,6 +1371,9 @@ chat_input_key_press_event_cb (GtkWidget   *widget,
 		gtk_adjustment_set_value (adj, val);
 		return TRUE;
 	}
+	if (event->keyval == GDK_Escape) {
+		empathy_search_bar_hide (EMPATHY_SEARCH_BAR (priv->search_bar));
+	}
 	if (!(event->state & (GDK_CONTROL_MASK | GDK_SHIFT_MASK)) &&
 	    event->keyval == GDK_Tab) {
 		GtkTextBuffer *buffer;
@@ -1337,7 +1387,9 @@ chat_input_key_press_event_cb (GtkWidget   *widget,
 
 		/* Get the start of the nick to complete. */
 		gtk_text_buffer_get_iter_at_mark (buffer, &start, gtk_text_buffer_get_insert (buffer));
-		gtk_text_iter_backward_word_start (&start);
+		if (gtk_text_iter_backward_find_char (&start, &empathy_isspace_cb, NULL, NULL)) {
+			gtk_text_iter_set_offset (&start, gtk_text_iter_get_offset (&start) + 1);
+		}
 		is_start_of_buffer = gtk_text_iter_is_start (&start);
 
 		list = empathy_contact_list_get_members (EMPATHY_CONTACT_LIST (priv->tp_chat));
@@ -1354,6 +1406,8 @@ chat_input_key_press_event_cb (GtkWidget   *widget,
 			guint        len;
 			const gchar *text;
 			gchar       *complete_char = NULL;
+			GString     *message = NULL;
+			GList       *l;
 
 			gtk_text_buffer_delete (buffer, &start, &current);
 
@@ -1369,6 +1423,18 @@ chat_input_key_press_event_cb (GtkWidget   *widget,
 				text = empathy_contact_get_name (completed_list->data);
 			} else {
 				text = completed;
+
+				/* Print all hits to the scrollback view, so the
+				 * user knows what possibilities he has.
+				 * Fixes #599779
+				 * */
+				 message = g_string_new ("");
+				 for (l = completed_list; l != NULL; l = l->next) {
+					g_string_append (message, empathy_contact_get_name (l->data));
+					g_string_append (message, " - ");
+				 }
+				 empathy_chat_view_append_event (chat->view, message->str);
+				 g_string_free (message, TRUE);
 			}
 
 			gtk_text_buffer_insert_at_cursor (buffer, text, strlen (text));
@@ -1847,18 +1913,20 @@ chat_reset_size_request (gpointer widget)
 }
 
 static void
-chat_update_contacts_visibility (EmpathyChat *chat)
+chat_update_contacts_visibility (EmpathyChat *chat,
+			 gboolean show)
 {
 	EmpathyChatPriv *priv = GET_PRIV (chat);
-	gboolean show;
 	GtkAllocation allocation;
 
-	show = priv->remote_contact == NULL && priv->show_contacts;
-
 	if (!priv->scrolled_window_contacts) {
 		return;
 	}
 
+	if (priv->remote_contact != NULL) {
+		show = FALSE;
+	}
+
 	if (show && priv->contact_list_view == NULL) {
 		EmpathyContactListStore *store;
 		gint                     min_width;
@@ -1909,7 +1977,7 @@ empathy_chat_set_show_contacts (EmpathyChat *chat,
 
 	priv->show_contacts = show;
 
-	chat_update_contacts_visibility (chat);
+	chat_update_contacts_visibility (chat, show);
 
 	g_object_notify (G_OBJECT (chat), "show-contacts");
 }
@@ -1939,7 +2007,7 @@ chat_remote_contact_changed_cb (EmpathyChat *chat)
 		g_object_get (channel, "handle-type", &priv->handle_type, NULL);
 	}
 
-	chat_update_contacts_visibility (chat);
+	chat_update_contacts_visibility (chat, priv->show_contacts);
 
 	g_object_notify (G_OBJECT (chat), "remote-contact");
 	g_object_notify (G_OBJECT (chat), "id");
@@ -1964,9 +2032,22 @@ chat_destroy_cb (EmpathyTpChat *tp_chat,
 
 	empathy_chat_view_append_event (chat->view, _("Disconnected"));
 	gtk_widget_set_sensitive (chat->input_text_view, FALSE);
-	empathy_chat_set_show_contacts (chat, FALSE);
+
+	chat_update_contacts_visibility (chat, FALSE);
 }
 
+static gboolean
+chat_hpaned_pos_changed_cb (GtkWidget* hpaned, gpointer user_data)
+{
+	gint hpaned_pos;
+	hpaned_pos = gtk_paned_get_position (GTK_PANED(hpaned));
+	empathy_conf_set_int (empathy_conf_get (),
+			      EMPATHY_PREFS_UI_CHAT_WINDOW_PANED_POS,
+			      hpaned_pos);
+	return TRUE;
+}
+
+
 static void
 show_pending_messages (EmpathyChat *chat) {
 	EmpathyChatPriv *priv = GET_PRIV (chat);
@@ -1975,6 +2056,9 @@ show_pending_messages (EmpathyChat *chat) {
 	if (chat->view == NULL || priv->tp_chat == NULL)
 		return;
 
+	if (!priv->can_show_pending)
+		return;
+
 	messages = empathy_tp_chat_get_pending_messages (priv->tp_chat);
 
 	for (l = messages; l != NULL ; l = g_list_next (l)) {
@@ -1992,6 +2076,7 @@ chat_create_ui (EmpathyChat *chat)
  	GList           *list = NULL;
 	gchar           *filename;
 	GtkTextBuffer   *buffer;
+	gint              paned_pos;
 
 	filename = empathy_file_lookup ("empathy-chat.ui",
 					"libempathy-gtk");
@@ -2055,11 +2140,30 @@ chat_create_ui (EmpathyChat *chat)
 			   chat->input_text_view);
 	gtk_widget_show (chat->input_text_view);
 
+	/* Add the (invisible) search bar */
+	priv->search_bar = empathy_search_bar_new (chat->view);
+	gtk_box_pack_start (GTK_BOX(priv->vbox_left),
+	                    priv->search_bar,
+	                    FALSE, FALSE, 0);
+	gtk_box_reorder_child (GTK_BOX(priv->vbox_left), priv->search_bar, 1);
+
 	/* Initialy hide the topic, will be shown if not empty */
 	gtk_widget_hide (priv->hbox_topic);
 
+	g_signal_connect (priv->hpaned, "notify::position",
+			  G_CALLBACK (chat_hpaned_pos_changed_cb),
+			  NULL);
+
+        /* Load the paned position */
+	if (empathy_conf_get_int (empathy_conf_get (),
+				 EMPATHY_PREFS_UI_CHAT_WINDOW_PANED_POS,
+				 &paned_pos)
+		&& paned_pos)
+		gtk_paned_set_position (GTK_PANED(priv->hpaned), paned_pos);
+
 	/* Set widget focus order */
-	list = g_list_append (NULL, priv->scrolled_window_input);
+	list = g_list_append (NULL, priv->search_bar);
+	list = g_list_append (list, priv->scrolled_window_input);
 	gtk_container_set_focus_chain (GTK_CONTAINER (priv->vbox_left), list);
 	g_list_free (list);
 
@@ -2187,8 +2291,11 @@ static void
 chat_constructed (GObject *object)
 {
 	EmpathyChat *chat = EMPATHY_CHAT (object);
+	EmpathyChatPriv *priv = GET_PRIV (chat);
 
-	chat_add_logs (chat);
+	if (priv->handle_type != TP_HANDLE_TYPE_ROOM)
+		chat_add_logs (chat);
+	priv->can_show_pending = TRUE;
 	show_pending_messages (chat);
 }
 
@@ -2781,6 +2888,18 @@ empathy_chat_paste (EmpathyChat *chat)
 	gtk_text_buffer_paste_clipboard (buffer, clipboard, NULL, TRUE);
 }
 
+void
+empathy_chat_find (EmpathyChat *chat)
+{
+	EmpathyChatPriv *priv;
+
+	g_return_if_fail (EMPATHY_IS_CHAT (chat));
+
+	priv = GET_PRIV (chat);
+
+	empathy_search_bar_show (EMPATHY_SEARCH_BAR (priv->search_bar));
+}
+
 void
 empathy_chat_correct_word (EmpathyChat  *chat,
 			  GtkTextIter *start,
