@@ -35,13 +35,17 @@
 #include <gtk/gtk.h>
 
 #include <libempathy/empathy-contact.h>
-#include <libempathy/empathy-account.h>
 #include <libempathy/empathy-ft-handler.h>
 
 #include "empathy-chat-view.h"
 
 G_BEGIN_DECLS
 
+#define EMPATHY_RECT_IS_ON_SCREEN(x,y,w,h) ((x) + (w) > 0 && \
+					    (y) + (h) > 0 && \
+					    (x) < gdk_screen_width () && \
+					    (y) < gdk_screen_height ())
+
 void            empathy_gtk_init                        (void);
 GRegex *        empathy_uri_regex_dup_singleton         (void);
 
@@ -59,6 +63,7 @@ GtkWidget     *empathy_builder_unref_and_keep_widget    (GtkBuilder       *gui,
 /* Pixbufs */
 const gchar * empathy_icon_name_for_presence            (TpConnectionPresenceType  presence);
 const gchar * empathy_icon_name_for_contact             (EmpathyContact   *contact);
+const gchar * empathy_protocol_name_for_contact         (EmpathyContact   *contact);
 GdkPixbuf *   empathy_pixbuf_from_data                  (gchar            *data,
 							 gsize             data_size);
 GdkPixbuf *   empathy_pixbuf_from_data_and_mime         (gchar            *data,
@@ -70,6 +75,14 @@ GdkPixbuf *   empathy_pixbuf_from_avatar_scaled         (EmpathyAvatar    *avata
 GdkPixbuf *   empathy_pixbuf_avatar_from_contact_scaled (EmpathyContact   *contact,
 							 gint              width,
 							 gint              height);
+GdkPixbuf *   empathy_pixbuf_protocol_from_contact_scaled (EmpathyContact   *contact,
+							 gint              width,
+							 gint              height);
+GdkPixbuf *   empathy_pixbuf_contact_status_icon (EmpathyContact   *contact,
+							 gboolean          show_protocol);
+GdkPixbuf *   empathy_pixbuf_contact_status_icon_with_icon_name (EmpathyContact   *contact,
+							 const gchar       *icon_name,
+							 gboolean          show_protocol);
 GdkPixbuf *   empathy_pixbuf_scale_down_if_necessary    (GdkPixbuf        *pixbuf,
 							 gint              max_size);
 GdkPixbuf *   empathy_pixbuf_from_icon_name             (const gchar      *icon_name,
@@ -106,6 +119,10 @@ void        empathy_toggle_button_set_state_quietly     (GtkWidget        *widge
 GtkWidget * empathy_link_button_new                     (const gchar      *url,
 							 const gchar      *title);
 
+void        empathy_send_file                           (EmpathyContact   *contact,
+							 GFile            *file);
+void        empathy_send_file_from_uri_list             (EmpathyContact   *contact,
+							 const gchar      *uri_list);
 void        empathy_send_file_with_file_chooser         (EmpathyContact   *contact);
 void        empathy_receive_file_with_file_chooser      (EmpathyFTHandler *handler);
 
@@ -113,6 +130,51 @@ gchar *     empathy_make_absolute_url                   (const gchar *url);
 gchar *     empathy_make_absolute_url_len               (const gchar *url,
 							 guint len);
 
+/* String parser */
+typedef struct _EmpathyStringParser EmpathyStringParser;
+
+typedef void (*EmpathyStringReplace) (const gchar *text,
+				      gssize len,
+				      gpointer match_data,
+				      gpointer user_data);
+typedef void (*EmpathyStringMatch) (const gchar *text,
+				    gssize len,
+				    EmpathyStringReplace replace_func,
+				    EmpathyStringParser *sub_parsers,
+				    gpointer user_data);
+
+struct _EmpathyStringParser {
+	EmpathyStringMatch match_func;
+	EmpathyStringReplace replace_func;
+};
+
+void
+empathy_string_parser_substr (const gchar *text,
+			      gssize len,
+			      EmpathyStringParser *parsers,
+			      gpointer user_data);
+
+void
+empathy_string_match_link (const gchar *text,
+			   gssize len,
+			   EmpathyStringReplace replace_func,
+			   EmpathyStringParser *sub_parsers,
+			   gpointer user_data);
+
+void
+empathy_string_match_smiley (const gchar *text,
+			     gssize len,
+			     EmpathyStringReplace replace_func,
+			     EmpathyStringParser *sub_parsers,
+			     gpointer user_data);
+
+void
+empathy_string_match_all (const gchar *text,
+			  gssize len,
+			  EmpathyStringReplace replace_func,
+			  EmpathyStringParser *sub_parsers,
+			  gpointer user_data);
+
 G_END_DECLS
 
 #endif /*  __EMPATHY_UI_UTILS_H__ */
