@@ -201,8 +201,8 @@ empathy_account_settings_constructed (GObject *object)
     {
       tp_account_prepare_async (priv->account, NULL,
           empathy_account_settings_account_ready_cb, self);
-      g_signal_connect (priv->managers, "notify::ready",
-        G_CALLBACK (empathy_account_settings_managers_ready_cb), self);
+      empathy_signal_connect_weak (priv->managers, "notify::ready",
+        G_CALLBACK (empathy_account_settings_managers_ready_cb), object);
     }
 
   if (G_OBJECT_CLASS (
@@ -1283,3 +1283,11 @@ empathy_account_settings_is_valid (EmpathyAccountSettings *settings)
 
   return !missed;
 }
+
+const TpConnectionManagerProtocol *
+empathy_account_settings_get_tp_protocol (EmpathyAccountSettings *self)
+{
+  EmpathyAccountSettingsPriv *priv = GET_PRIV (self);
+
+  return tp_connection_manager_get_protocol (priv->manager, priv->protocol);
+}
