@@ -15,9 +15,9 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- * 
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ *
  * Authors: Martyn Russell <martyn@imendio.com>
  *          Xavier Claessens <xclaesse@gmail.com>
  */
@@ -25,9 +25,7 @@
 #include <config.h>
 
 #include <glib/gi18n.h>
-#include <gtk/gtkaboutdialog.h>
-#include <gtk/gtksizegroup.h>
-#include <glade/glade.h>
+#include <gtk/gtk.h>
 
 #include <libempathy-gtk/empathy-ui-utils.h>
 
@@ -45,6 +43,7 @@ static const char *authors[] = {
 	"Aurelien Naldi",
 	"Bastien Nocera",
 	"Christoffer Olsen",
+	"Cosimo Cecchi",
 	"Elliot Fairweather",
 	"Frederic Crozat",
 	"Frederic Peters",
@@ -77,6 +76,7 @@ static const char *documenters[] = {
 static const char *artists[] = {
 	"Andreas Nilsson <nisses.mail@home.se>",
 	"Vinicius Depizzol <vdepizzol@gmail.com>",
+	"K.Vishnoo Charan Reddy <drkvi-a@yahoo.com>",
 	NULL
 };
 
