@@ -43,6 +43,7 @@
 #include <libempathy-gtk/empathy-video-src.h>
 #include <libempathy-gtk/empathy-ui-utils.h>
 #include <libempathy-gtk/empathy-sound.h>
+#include <libempathy-gtk/empathy-geometry.h>
 
 #define DEBUG_FLAG EMPATHY_DEBUG_VOIP
 #include <libempathy/empathy-debug.h>
@@ -725,7 +726,7 @@ empathy_call_window_set_state_connecting (EmpathyCallWindow *window)
 {
   EmpathyCallWindowPriv *priv = GET_PRIV (window);
 
-  empathy_call_window_status_message (window, _("Connecting..."));
+  empathy_call_window_status_message (window, _("Connecting…"));
   priv->call_state = CONNECTING;
 
   if (priv->outgoing)
@@ -1076,6 +1077,8 @@ empathy_call_window_init (EmpathyCallWindow *self)
 
   g_object_ref (priv->ui_manager);
   g_object_unref (gui);
+
+  empathy_geometry_bind (GTK_WINDOW (self), "call-window");
 }
 
 /* Instead of specifying a width and a height, we specify only one size. That's
@@ -1228,19 +1231,13 @@ empathy_call_window_constructed (GObject *object)
   empathy_call_window_setup_avatars (self, priv->handler);
   empathy_call_window_set_state_connecting (self);
 
-  if (empathy_call_handler_has_initial_video (priv->handler))
-    {
-      /* Enable 'send video' buttons and display the preview */
-      gtk_toggle_tool_button_set_active (
-          GTK_TOGGLE_TOOL_BUTTON (priv->tool_button_camera_on), TRUE);
-
-      display_video_preview (self, TRUE);
-    }
-  else
+  if (!empathy_call_handler_has_initial_video (priv->handler))
     {
       gtk_toggle_tool_button_set_active (
           GTK_TOGGLE_TOOL_BUTTON (priv->tool_button_camera_off), TRUE);
     }
+  /* If call has InitialVideo, the preview will be started once the call has
+   * been started (start_call()). */
 }
 
 static void empathy_call_window_dispose (GObject *object);
@@ -1435,7 +1432,7 @@ empathy_call_window_request_resource_cb (EmpathyCallHandler *handler,
   EmpathyCallWindow *self = EMPATHY_CALL_WINDOW (user_data);
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
 
-  if (type != TP_MEDIA_STREAM_TYPE_VIDEO)
+  if (type != FS_MEDIA_TYPE_VIDEO)
     return TRUE;
 
   if (direction == FS_DIRECTION_RECV)
@@ -1827,7 +1824,7 @@ media_stream_error_to_txt (EmpathyCallWindow *self,
             "product=Telepathy&amp;component=%s", cm);
 
         result = g_strdup_printf (
-            _("Something not expected happened in a Telepathy component. "
+            _("Something unexpected happened in a Telepathy component. "
               "Please <a href=\"%s\">report this bug</a> and attach "
               "logs gathered from the 'Debug' window in the Help menu."), url);
 
@@ -2051,6 +2048,22 @@ empathy_call_window_remove_video_input (EmpathyCallWindow *self)
   gtk_widget_show (priv->self_user_avatar_widget);
 }
 
+static void
+start_call (EmpathyCallWindow *self)
+{
+  EmpathyCallWindowPriv *priv = GET_PRIV (self);
+
+  priv->call_started = TRUE;
+  empathy_call_handler_start_call (priv->handler);
+  gst_element_set_state (priv->pipeline, GST_STATE_PLAYING);
+
+  if (empathy_call_handler_has_initial_video (priv->handler))
+    {
+      /* Enable 'send video' buttons and display the preview */
+      gtk_toggle_tool_button_set_active (
+          GTK_TOGGLE_TOOL_BUTTON (priv->tool_button_camera_on), TRUE);
+    }
+}
 
 static gboolean
 empathy_call_window_bus_message (GstBus *bus, GstMessage *message,
@@ -2077,9 +2090,7 @@ empathy_call_window_bus_message (GstBus *bus, GstMessage *message,
             gst_message_parse_state_changed (message, NULL, &newstate, NULL);
             if (newstate == GST_STATE_PAUSED)
               {
-                priv->call_started = TRUE;
-                empathy_call_handler_start_call (priv->handler);
-                gst_element_set_state (priv->pipeline, GST_STATE_PLAYING);
+                start_call (self);
               }
           }
         break;
@@ -2473,10 +2484,8 @@ empathy_call_window_restart_call (EmpathyCallWindow *window)
   priv->outgoing = TRUE;
   empathy_call_window_set_state_connecting (window);
 
-  priv->call_started = TRUE;
-  empathy_call_handler_start_call (priv->handler);
+  start_call (window);
   empathy_call_window_setup_avatars (window, priv->handler);
-  gst_element_set_state (priv->pipeline, GST_STATE_PLAYING);
 
   gtk_action_set_sensitive (priv->redial, FALSE);
   gtk_widget_set_sensitive (priv->redial_button, FALSE);
@@ -2621,7 +2630,7 @@ block_camera_control_signals (EmpathyCallWindow *self)
   g_signal_handlers_block_by_func (priv->tool_button_camera_on,
       tool_button_camera_on_toggled_cb, self);
   g_signal_handlers_block_by_func (priv->action_camera,
-      tool_button_camera_on_toggled_cb, self);
+      action_camera_change_cb, self);
 }
 
 static void
@@ -2636,5 +2645,5 @@ unblock_camera_control_signals (EmpathyCallWindow *self)
   g_signal_handlers_unblock_by_func (priv->tool_button_camera_on,
       tool_button_camera_on_toggled_cb, self);
   g_signal_handlers_unblock_by_func (priv->action_camera,
-      tool_button_camera_on_toggled_cb, self);
+      action_camera_change_cb, self);
 }
