@@ -93,6 +93,7 @@ typedef struct {
 	GtkAction   *menu_edit_cut;
 	GtkAction   *menu_edit_copy;
 	GtkAction   *menu_edit_paste;
+	GtkAction   *menu_edit_find;
 
 	GtkAction   *menu_tabs_next;
 	GtkAction   *menu_tabs_prev;
@@ -252,9 +253,9 @@ chat_window_create_label (EmpathyChatWindow *window,
 
 	gtk_misc_set_padding (GTK_MISC (name_label), 2, 0);
 	gtk_misc_set_alignment (GTK_MISC (name_label), 0.0, 0.5);
-	g_object_set_data (G_OBJECT (chat),
+	g_object_set_data_full (G_OBJECT (chat),
 		is_tab_label ? "chat-window-tab-label" : "chat-window-menu-label",
-		name_label);
+		g_object_ref (name_label), g_object_unref);
 
 	status_image = gtk_image_new ();
 
@@ -264,12 +265,12 @@ chat_window_create_label (EmpathyChatWindow *window,
 	gtk_box_pack_start (GTK_BOX (event_box_hbox), status_image, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (event_box_hbox), name_label, TRUE, TRUE, 0);
 
-	g_object_set_data (G_OBJECT (chat),
+	g_object_set_data_full (G_OBJECT (chat),
 		is_tab_label ? "chat-window-tab-image" : "chat-window-menu-image",
-		status_image);
-	g_object_set_data (G_OBJECT (chat),
+		g_object_ref (status_image), g_object_unref);
+	g_object_set_data_full (G_OBJECT (chat),
 		is_tab_label ? "chat-window-tab-tooltip-widget" : "chat-window-menu-tooltip-widget",
-		event_box);
+		g_object_ref (event_box), g_object_unref);
 
 	gtk_container_add (GTK_CONTAINER (event_box), event_box_hbox);
 	gtk_box_pack_start (GTK_BOX (hbox), event_box, TRUE, TRUE, 0);
@@ -277,7 +278,8 @@ chat_window_create_label (EmpathyChatWindow *window,
 	if (is_tab_label) {
 		close_button = gtk_button_new ();
 		gtk_button_set_relief (GTK_BUTTON (close_button), GTK_RELIEF_NONE);
-		g_object_set_data (G_OBJECT (chat), "chat-window-tab-close-button", close_button);
+		g_object_set_data_full (G_OBJECT (chat), "chat-window-tab-close-button",
+			g_object_ref (close_button), g_object_unref);
 
 		/* We don't want focus/keynav for the button to avoid clutter, and
 		 * Ctrl-W works anyway.
@@ -1000,6 +1002,19 @@ chat_window_paste_activate_cb (GtkAction         *action,
 	empathy_chat_paste (priv->current_chat);
 }
 
+static void
+chat_window_find_activate_cb (GtkAction         *action,
+			      EmpathyChatWindow *window)
+{
+	EmpathyChatWindowPriv *priv;
+
+	g_return_if_fail (EMPATHY_IS_CHAT_WINDOW (window));
+
+	priv = GET_PRIV (window);
+
+	empathy_chat_find (priv->current_chat);
+}
+
 static void
 chat_window_tabs_next_activate_cb (GtkAction         *action,
 				   EmpathyChatWindow *window)
@@ -1804,6 +1819,7 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 				       "menu_edit_cut", &priv->menu_edit_cut,
 				       "menu_edit_copy", &priv->menu_edit_copy,
 				       "menu_edit_paste", &priv->menu_edit_paste,
+				       "menu_edit_find", &priv->menu_edit_find,
 				       "menu_tabs_next", &priv->menu_tabs_next,
 				       "menu_tabs_prev", &priv->menu_tabs_prev,
 				       "menu_tabs_left", &priv->menu_tabs_left,
@@ -1823,6 +1839,7 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 			      "menu_edit_cut", "activate", chat_window_cut_activate_cb,
 			      "menu_edit_copy", "activate", chat_window_copy_activate_cb,
 			      "menu_edit_paste", "activate", chat_window_paste_activate_cb,
+			      "menu_edit_find", "activate", chat_window_find_activate_cb,
 			      "menu_tabs_next", "activate", chat_window_tabs_next_activate_cb,
 			      "menu_tabs_prev", "activate", chat_window_tabs_previous_activate_cb,
 			      "menu_tabs_left", "activate", chat_window_tabs_left_activate_cb,
